"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = require("ava");
const prray_1 = require("../src/prray");
ava_1.default('Compatibility with array: indexing', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = [1, 2, 3, 4];
    const prr = new prray_1.default(1, 2, 3, 4);
    t.is(prr.length, arr.length);
    t.is(prr[1], arr[1]);
    t.is(prr[prr.length - 1], arr[arr.length - 1]);
}));
ava_1.default('Compatibility with array: stringify', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = [1, 2, 3, 4];
    const prr = new prray_1.default(1, 2, 3, 4);
    t.is(prr.toString(), arr.toString());
    t.is(prr.toLocaleString(), arr.toLocaleString());
    t.is(JSON.stringify(prr), JSON.stringify(arr));
}));
ava_1.default('Compatibility with array: prototype', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = [1, 2, 3, 4];
    const prr = new prray_1.default(1, 2, 3, 4);
    t.is(prr instanceof prray_1.default, true);
    t.is(prr instanceof Array, true);
    t.is(prr.__proto__.__proto__, arr.__proto__);
    t.is(Array.isArray(prr), true);
}));
ava_1.default('Compatibility with array: spread operator', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = [1, 2, 3, 4];
    const prr = new prray_1.default(1, 2, 3, 4);
    t.deepEqual([...prr], [...arr]);
}));
ava_1.default('Compatibility with array: for-index loop', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = ['a', 'b', 'c', 'd'];
    const prr = new prray_1.default('a', 'b', 'c', 'd');
    for (let ix = 0; ix < arr.length; ix++) {
        t.is(prr[ix], arr[ix]);
    }
}));
ava_1.default('Compatibility with array: for-of loop', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = ['a', 'b', 'c', 'd'];
    const prr = new prray_1.default('a', 'b', 'c', 'd');
    let ix = 0;
    for (const v of prr) {
        t.is(v, arr[ix++]);
    }
}));
ava_1.default('Compatibility with array: for-in loop', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = ['a', 'b', 'c', 'd'];
    const prr = new prray_1.default('a', 'b', 'c', 'd');
    for (const key in prr) {
        t.is(prr[key], arr[key]);
    }
}));
ava_1.default('Compatibility with array: iteration', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const arr = ['a', 'b', 'c', 'd'];
    const prr = new prray_1.default('a', 'b', 'c', 'd');
    const pi = prr[Symbol.iterator]();
    const ai = arr[Symbol.iterator]();
    t.is(pi.next().value, ai.next().value);
    t.is(pi.next().done, ai.next().done);
    t.is(pi.next().value, ai.next().value);
    t.is(pi.next().done, ai.next().done);
    t.is(pi.next().value, ai.next().value);
    t.is(pi.next().done, ai.next().done);
    t.is(pi.next().value, ai.next().value);
    t.is(pi.next().done, ai.next().done);
}));
