"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = require("ava");
const sinon = require("sinon");
const prray_1 = require("../src/prray");
const test_utils_1 = require("./test-utils");
ava_1.default('prray forEachAsync', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const result1 = [];
    yield prray_1.default.from([1, 2, 3]).forEachAsync((v) => __awaiter(void 0, void 0, void 0, function* () {
        yield test_utils_1.delay(100);
        result1.push(v + 1);
    }));
    t.deepEqual(result1, [2, 3, 4]);
    const result2 = [];
    yield prray_1.default.from([1, 2, 3]).forEachAsync(v => {
        result2.push(v + 1);
    });
    t.deepEqual(result2, [2, 3, 4]);
    yield prray_1.default.from([1, 2, 3]).forEachAsync((v) => __awaiter(void 0, void 0, void 0, function* () {
        yield test_utils_1.delay(100);
        result1.push(v + 1);
    }), { concurrency: 1 });
}));
ava_1.default('prraypromise forEachAsync', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const result1 = [];
    yield test_utils_1.toPrrayPromise([1, 2, 3]).forEachAsync((v) => __awaiter(void 0, void 0, void 0, function* () {
        yield test_utils_1.delay(100);
        result1.push(v + 1);
    }));
    t.deepEqual(result1, [2, 3, 4]);
    const result2 = [];
    yield test_utils_1.toPrrayPromise([1, 2, 3]).forEachAsync(v => {
        result2.push(v + 1);
    });
    t.deepEqual(result2, [2, 3, 4]);
}));
ava_1.default('prray forEachAsync compatibility', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const func = sinon.fake();
    const prr = prray_1.default.from(['a', 'b', 'c']);
    yield prr.forEachAsync(func);
    t.is(func.called, true);
    t.is(func.callCount, 3);
    t.is(func.args[0][0], 'a');
    t.is(func.args[0][1], 0);
    t.is(func.args[0][2], prr);
    t.is(func.args[1][0], 'b');
    t.is(func.args[1][1], 1);
    t.is(func.args[1][2], prr);
    t.is(func.args[2][0], 'c');
    t.is(func.args[2][1], 2);
    t.is(func.args[2][2], prr);
}));
ava_1.default('prraypromise forEachAsync compatibility', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const func = sinon.fake();
    const p = test_utils_1.toPrrayPromise(['a', 'b', 'c']);
    yield p.forEachAsync(func);
    t.is(func.called, true);
    t.is(func.callCount, 3);
    t.is(func.args[0][0], 'a');
    t.is(func.args[0][1], 0);
    t.deepEqual(func.args[0][2], prray_1.default.from(['a', 'b', 'c']));
    t.is(func.args[1][0], 'b');
    t.is(func.args[1][1], 1);
    t.deepEqual(func.args[1][2], prray_1.default.from(['a', 'b', 'c']));
    t.is(func.args[2][0], 'c');
    t.is(func.args[2][1], 2);
    t.deepEqual(func.args[2][2], prray_1.default.from(['a', 'b', 'c']));
    yield p.forEachAsync(func, { concurrency: 2 });
}));
