"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = require("ava");
const sinon = require("sinon");
const prray_1 = require("../src/prray");
const prraypromise_1 = require("../src/prraypromise");
const test_utils_1 = require("./test-utils");
ava_1.default('prray map', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const p = prray_1.default.from([1, 2, 3]);
    const result = p.map(test_utils_1.addOne);
    t.true(result instanceof prray_1.default);
    t.deepEqual(result, prray_1.default.from([2, 3, 4]));
}));
ava_1.default('prray map with async callback', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const p = prray_1.default.from([1, 2, 3]);
    const result = p.map(test_utils_1.addOneAsync);
    t.true(result instanceof prray_1.default);
    t.true(result[0] instanceof Promise);
    t.deepEqual(yield Promise.all(result), [2, 3, 4]);
}));
ava_1.default('prraypromise map', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const pp = test_utils_1.toPrrayPromise([1, 2, 3]);
    const result = pp.map(test_utils_1.addOne);
    t.true(result instanceof prraypromise_1.PrrayPromise);
    t.deepEqual(yield result, prray_1.default.from([2, 3, 4]));
}));
ava_1.default('prraypromise map with async callback', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const p = test_utils_1.toPrrayPromise([1, 2, 3]);
    const result = p.map(test_utils_1.addOneAsync);
    t.true(result instanceof prraypromise_1.PrrayPromise);
    const resolved = yield result;
    t.true(resolved[0] instanceof Promise);
    t.deepEqual(yield Promise.all(resolved), [2, 3, 4]);
}));
ava_1.default('prray map detail', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const prr = prray_1.default.from(['a', 'b', 'c']);
    const func = sinon.fake();
    prr.map(func);
    t.is(func.called, true);
    t.is(func.callCount, 3);
    t.is(func.args[0][0], 'a');
    t.is(func.args[0][1], 0);
    t.is(func.args[0][2], prr);
    t.is(func.args[1][0], 'b');
    t.is(func.args[1][1], 1);
    t.is(func.args[1][2], prr);
    t.is(func.args[2][0], 'c');
    t.is(func.args[2][1], 2);
    t.is(func.args[2][2], prr);
}));
ava_1.default('prraypromise map detail', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const prr = prray_1.default.from(['a', 'b', 'c']);
    const prom = test_utils_1.toPrrayPromise(prr);
    const func = sinon.fake();
    yield prom.map(func);
    t.is(func.called, true);
    t.is(func.callCount, 3);
    t.is(func.args[0][0], 'a');
    t.is(func.args[0][1], 0);
    t.is(func.args[0][2], prr);
    t.is(func.args[1][0], 'b');
    t.is(func.args[1][1], 1);
    t.is(func.args[1][2], prr);
    t.is(func.args[2][0], 'c');
    t.is(func.args[2][1], 2);
    t.is(func.args[2][2], prr);
}));
